/* six-digit-elapsed-remaining.m by Ariszló <ariszlo@gmail.com> (2024-01-06)

Show elapsed time in one text object and remaining time in another one.
If the song is longer than an hour then show elapsed/remaining time in
HH:MM:SS format. If shorter then in MM:SS.CC format, where CC = centiseconds.

This script requires the following text objects (replace the question marks
with actual values):

  <text
    id="Elapsed Time"
    x="?" y="?" w="?" h="?"
  />
  <text
    id="Remaining Time"
    x="?" y="?" w="?" h="?
  />

Call the script like this:

  <script id="timers" file="scripts/six-digit-elapsed-remaining.maki"/> 
*/

// Never forget to include std.mi:
#include "lib/std.mi"

// Global variables used throughout this script:
Global Text elapsedText, remainingText;
Global Timer trackTimer;

// Do this when the script is loaded:
System.onScriptLoaded() {
  // Get the xml group calling this script:
  Group ScriptGrp = getScriptGroup();

  // Find the xml objects within the group:
  elapsedText = ScriptGrp.findObject("Elapsed Time");
  remainingText = ScriptGrp.findObject("Remaining Time");

  // Initialize the script’s timer:
  trackTimer = new Timer;
  trackTimer.setDelay(20);

  // Start the timer if playback not stopped:
  if (getStatus() != 0) trackTimer.start();
}

// Start the timer when playback starts or resumes:
System.onPlay() {
  trackTimer.start();
}

System.onResume() {
  trackTimer.start();
}

// Stop the timer when playback is stopped:
System.onStop() {
  trackTimer.stop();
  elapsedText.setText("");
  remainingText.setText("");
}

// Show elapsed and remaining time while the timer is running:
trackTimer.onTimer() {
  // Define a *local* variable for elapsed time:
  int elt = getPosition();
  // Get elapsed centiseconds to show after the decimal dot:
  String elapsedCents = getToken((floatToString(elt/1000, 2)), "." , 1);

  // == Elapsed Time ==
  // Add a leading 0 if the track is shorter than 10 minutes and show centiseconds after a dot:
  if (getPlayItemLength() < 600000) {
    elapsedText.setText("0" + integerToTime(elt) + "." + elapsedCents);
  }
  // This is for tracks longer than 10 minutes but shorter than an hour:
  else if (getPlayItemLength() < 3600000) {
    if (getPosition() < 600000) {
      elapsedText.setText("0" + integerToTime(elt) + "." + elapsedCents);
    }
    else {
      elapsedText.setText(integerToTime(elt) + "." + elapsedCents);
    }
  }
  // This is for tracks longer than an hour but shorter than 10 hours:
  else if (getPlayItemLength() < 36000000) {
    elapsedText.setText("0" + integerToLongTime(elt));
  }
  // This is for tracks longer than 10 hours:
  else {
    if (getPosition() < 36000000) {
      elapsedText.setText("0" + integerToLongTime(elt));
    }
    else {
      elapsedText.setText(integerToLongTime(elt));
    }
  }

  // == Remaining Time ==
  // Check if remaining time can be calculated from elapsed time and song length:
  if (getPlayItemLength() > 0) {
    // Define a local variable for remaining time:
    Int remt = getPlayItemLength()-getPosition();
    // Get remaining centiseconds to show after the decimal dot:
    String remainingCents = getToken((floatToString(remt/1000, 2)), "." , 1);

    // Add a leading -0 if the track is shorter than 10 minutes and show centiseconds after a dot:
    if (getPlayItemLength() < 600000) {
      remainingText.setText("-0" + integerToTime(remt) + "." + remainingCents);
    }
    // This is for tracks longer than 10 minutes but shorter than an hour:
    else if (getPlayItemLength() < 3600000) {
      if (remt < 600000) {
        remainingText.setText("-0" + integerToTime(remt) + "." + remainingCents);
      }
      else {
        remainingText.setText("-" + integerToTime(remt) + "." + remainingCents);
      }
    }
    // This is for tracks longer than an hour but shorter than 10 hours:
    else if (getPlayItemLength() < 36000000) {
      remainingText.setText("-0" + integerToLongTime(remt));
    }
    // This is for tracks longer than 10 hours:
    else {
      if (remt < 36000000) {
        remainingText.setText("-0" + integerToLongTime(remt));
      }
      else {
        remainingText.setText("-" + integerToLongTime(remt));
      }
    }
  }
  // Show an empty string for remaining time if it cannot be calculated:
  else {
    remainingText.setText("");
  }
}


// Do this before the script is unloaded:
System.onScriptUnloading() {
  trackTimer.stop();
  delete trackTimer;
}
